<?php
  /*
  /   Little Argon2 EXE Wrapper for PHP
  /   Implemented as a static class
  /   Works with initial release of Argon2
  /   By Jacob Allen
  */
  
  class Argon2{
    //Location of Argon2 EXE
    public static $Argon2_Path = "/includes/libs/";
    //Is the location relative
    public static $Argon2_Path_Relative = true;
    
    //Hash a password with specified salt and parameters, default values are suggested from specification
    public static function Hash($password, $salt, $t = 3, $m = 12, $p = 1, $d = false, $verbose = false){
      $descriptorspec = array(
         0 => array("pipe", "r"),
         1 => array("pipe", "w"),
         2 => array("pipe", "r")
      );
      
      $cwd = static::$Argon2_Path_Relative ? getcwd() . static::$Argon2_Path : static::$Argon2_Path;
      
      if(static::$Argon2_Path == "")
        $process = proc_open("argon2 ".escapeshellarg($salt)." -m $m -t $t -p $p ".($d ? "-d" : ""), $descriptorspec, $pipes);
      else
        $process = proc_open("argon2 ".escapeshellarg($salt)." -m $m -t $t -p $p ".($d ? "-d" : ""), $descriptorspec, $pipes, $cwd);
      
      if (is_resource($process)) {
        fwrite($pipes[0], $password);
        fclose($pipes[0]);
        $response = stream_get_contents($pipes[1]);
        fclose($pipes[1]);
        $errors = stream_get_contents($pipes[2]);
        fclose($pipes[2]);
        if(proc_close($process) != 0)
          throw new Exception("Argon2 process return non-zero response code.");
        if($errors != "")
          throw new Exception("Argon2 command error.");
      }else{
        throw new Exception("Failed to open argon2 process.");
      }
      
      $responselines = explode("\n", trim($response));
      if(!isset($responselines[7]) || $responselines[7] != "Verification ok"){
        throw new Exception("Argon2 verification not ok.");
      }
      
      $results = array(
        "type"          => trim(str_replace("Type:", "", $responselines[0])),
        "iterations"    => trim(str_replace("Iterations:", "", $responselines[1])),
        "memory"        => trim(str_replace("Memory:", "", $responselines[2])),
        "parallelism"   => trim(str_replace("Parallelism:", "", $responselines[3])),
        "hash"          => trim(str_replace("Hash:", "", $responselines[4])),
        "encoded"       => trim(str_replace("Encoded:", "", $responselines[5])),
        "seconds"       => trim(str_replace("seconds", "", $responselines[6])),
        "verification"  => trim(str_replace("Verification", "", $responselines[7]))
      );
      return $verbose ? $results : $results["encoded"];
    }
    
    //Verify a password against an encoded hash
    public static function Verify($password, $hash){
      $hash_array = explode('$', trim($hash,'$ \t\n\r\0\x0B'));
      
      $method  = $hash_array[0];
      $args    = $hash_array[1];
      $b64salt = $hash_array[2];
      $b64hash = $hash_array[3];
      
      if($method == "argon2i")
        $d = false;
      elseif ($method == "argon2d")
        $d = true;
      else
        throw new Exception("Unknown Argon2 hash method.");
      
      $exp_args = array();
      foreach(explode(',', $args) as $arg){
        $exp_arg = explode('=', $arg);
        $exp_args[$exp_arg[0]] = $exp_arg[1];
      }
      if(!(array_key_exists("t",$exp_args) && array_key_exists("m",$exp_args) && array_key_exists("p",$exp_args)))
        throw new Exception("Unknown Argon2 arguments method.");
      
      if(strlen($b64salt) != 22)
        throw new Exception("Salt too " . (strlen($b64salt) > 22 ? "long" : "short") . ".");
      
      if(strlen($b64hash) != 43)
        throw new Exception("Hash too " . (strlen($b64hash) > 43 ? "long" : "short") . ".");
      
      $newhash = static::Hash($password, trim(base64_decode($b64salt), "\x00"), $exp_args['t'], log($exp_args['m'], 2), $exp_args['p'], $d, false);
      
      return $hash == $newhash;
    }
    
    //Generate some salt, you can use a different functions as long as it's cryptographically secure
    public static function Gensalt($bytes = 10){
      return base64_encode(openssl_random_pseudo_bytes($bytes));
    }
  }
?>