<?php
  function user_exists($user){
    return is_dir("db/users/".sha1(strtolower($user)));
  }
  
  function user_has_icon($user){
    return is_file("db/users/".sha1(strtolower($user))."/icon.jpg");
  }  
  
  function message_exists($user, $id){
    return is_file("db/users/".sha1(strtolower($user))."/messages/".$id.".json");
  }
  
  function user_follows($user, $otheruser){
    $user = strtolower($user);
    $otheruser = strtolower($otheruser);
    $follows = get_user_data($user)->stream;
    return array_search($otheruser, $follows) !== false;
  }
  
  function get_user_data($user){
    $user = strtolower($user);
    $file = "db/users/".sha1($user)."/data.json";
    if(is_file($file)){
      $fp = fopen($file, "r");
      $data = fread($fp, filesize($file));
      fclose($fp);
      return json_decode($data);
    }
    return array();
  }
  
  function update_user_data($user, $data){
    $user = strtolower($user);
    $file = "db/users/".sha1($user)."/data.json";
    if(is_file($file)){
      $fp = fopen($file, "w");
      fwrite($fp, json_encode($data));
      fclose($fp);
      return true;
    }
    return false;
  }
  
  function create_message($name, $message){
    $display = $name;
    $name = strtolower($name);
    if(is_dir("db/users/".sha1($name)."/messages")){
      $fp = fopen("db/users/".sha1($name)."/messages/".sha1(uniqid('message_', true)).".json", "w+");
      fwrite($fp, json_encode(
        array(
          "display"=>$display,
          "message"=>$message,
          "time"=>time()
        )
      ));
      fclose($fp);
      return true;
    }else{
      return false;
    }
  }
  
  function how_many_stream_messages($name){
    $name = strtolower($name);
    $streams = get_user_data($name)->stream;
    $count = 0;
    foreach($streams as $stream){
      if(is_dir("db/users/".sha1($stream)."/messages")){
        $count += count(glob("db/users/".sha1($stream)."/messages/*.*"));
      }
    }
    return $count;
  }
  
  function how_many_messages($name){
    $name = strtolower($name);
    $count = 0;
    if(is_dir("db/users/".sha1($name)."/messages")){
      $count = count(glob("db/users/".sha1($name)."/messages/*.*"));
    }
    return $count;
  }
  
  function get_message($name, $message_id){
    $display = $name;
    $name = strtolower($name);
    if(is_dir("db/users/".sha1($name)."/messages")){
      $file = "db/users/".sha1($name)."/messages/".$message_id.".json";
      $fp = fopen($file, "r");
      $data = fread($fp, filesize($file));
      fclose($fp);
      return json_decode($data);
    }else{
      return false;
    }
  }
  
  function get_messages($name, $item_limit = -1, $offset = 0){
    $display = $name;
    $name = strtolower($name);
    if(is_dir("db/users/".sha1($name)."/messages")){
      $files = array();
      if ($handle = opendir("db/users/".sha1($name)."/messages")) {
        while(false !== ($file = readdir($handle))) {
          if($file != "." && $file != ".."){
            $files[filemtime("db/users/".sha1($name)."/messages/".$file).str_replace(".json", "", $file)] = $file;
          }
        }
        closedir($handle);
        ksort($files);
        $files = array_reverse($files);
        $results = array();
        $i = 0;
        foreach($files as $item){
          if($item_limit > 0 ? ($i >= $item_limit) : false)
            break;
          if($offset == 0){
            array_push($results, str_replace(".json", "", $item));
            $i += 1;
          }
          else $offset -= 1;
        }
        return $results;
      }else{
        return false;
      }
    }else{
      return false;
    }
  }
  
  function get_stream($name, $item_limit = -1, $offset = 0){
    $display = $name;
    $name = strtolower($name);
    $streams = get_user_data($name)->stream;
    $files = array();
    foreach($streams as $stream){
      if(is_dir("db/users/".sha1($stream)."/messages") && $handle = opendir("db/users/".sha1($stream)."/messages")){
        while(false !== ($file = readdir($handle))) {
          if($file != "." && $file != ".."){
            $index = filemtime("db/users/".sha1($stream)."/messages/".$file).str_replace(".json", "", $file);
            if(isset($files[$index])){
              $index += "-" . sha1($stream);
            }
            $files[$index] = array("user" => $stream, "message" => str_replace(".json", "", $file));
          }
        }
        closedir($handle);
      }else{
        $ud_to_update = get_user_data($name);
        $ud_to_update->stream = array_diff($ud_to_update->stream, array($stream));
        update_user_data($name, $ud_to_update);
      }
    }
    ksort($files);
    $files = array_reverse($files);
    $results = array();
    $i = 0;
    foreach($files as $item){
      if($item_limit > 0 ? ($i >= $item_limit) : false)
        break;
      if($offset == 0){
        array_push($results, $item);
        $i += 1;
      }
      else $offset -= 1;
    }
    return $results;
  }
  
  function create_user($name, $password){
    $display = $name;
    $name = strtolower($name);
    if(!is_dir("db/users/".sha1($name))){
      if(mkdir("db/users/".sha1($name))){
        mkdir("db/users/".sha1($name)."/messages");
        $fp = fopen("db/users/".sha1($name)."/data.json", "w+");
        $hash = "";
        if($HASH_TYPE == "argon2"){
          $hash = Argon2::Hash($password, Argon2::Gensalt());
        }else{
          $hash = password_hash($password, PASSWORD_DEFAULT);
        }
        fwrite($fp, json_encode(
          array(
            "display"=>$display,
            "has_icon"=>false,
            "description"=>"",
            "hash"=>$hash,
            "stream"=>array()
          )
        ));
        fclose($fp);
        return true;
      }else{
        return false;
      }
    }else{
      return false;
    }
  }
  
  function check_user($name, $password){
    $name = strtolower($name);
    if(is_dir("db/users/".sha1($name))){
      if(is_file("db/users/".sha1($name)."/data.json")){
        $udata = get_user_data($name);
        if(substr($udata->hash, 0, 8) == "$argon2i"){
          return Argon2::Verify($password, $udata->hash);
        }elseif(substr($udata->hash, 0, 3) == "$2y"){
          return password_verify($password, $udata->hash);
        }else{
          return false;
        }
      }else{
        return false;
      }
    }else{
      return false;
    }
  }
  
  function escapefordb($str){
    return get_magic_quotes_gpc() ? $str : addslashes($str);
  }
  
  function get_token(){
    return hash("tiger160,4", date('l jS A F Y h')."this is a very basic token, needs future improvement.");
  }
  
  function check_token($token){
    return hash("tiger160,4", date('l jS A F Y h')."this is a very basic token, needs future improvement.") == $token || hash("tiger160,4", date('l jS A F Y h', time() - (60*60))."this is a very basic token, needs future improvement.") == $token;
  }
?>